/* 

Copyright (c) 1999  Ulric Eriksson <ulric@siag.nu>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <stdio.h>
#include <stdlib.h>

#include <magick/api.h>
#include <magick/magick.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <xmagick.h>
#include "MagickP.h"

#define offset(field) XtOffsetOf(MagickRec, magick.field)
static XtResource resources[] = {
	{
		XtNmagick,		/* name */
		XtCMagick,		/* class */
		XtRPointer,		/* type */
		sizeof(XtPointer),	/* size */
		offset(image),		/* offset */
		XtRImmediate,		/* default_type */
		(XtPointer) NULL	/* default_addr */
	}
};
#undef offset

/* methods */
static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
static void Redisplay(Widget, XEvent *, Region);
static void Destroy(Widget);
static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);

/* actions */
static void MagickAction(Widget, XEvent *, String *, Cardinal *);

static XtActionsRec actions[] =
{
	{"magick", MagickAction},
};

/* translations */
static char translations[] =
"<Key>:		magick()\n";

MagickClassRec magickClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"Magick",
    /* widget_size		*/	sizeof(MagickRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	NULL,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	actions,
    /* num_actions		*/	XtNumber(actions),
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	Destroy,
    /* resize			*/	NULL,
    /* expose			*/	Redisplay,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	translations,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* magick fields */
    /* empty			*/	0
  }
};

WidgetClass magickWidgetClass = (WidgetClass)&magickClassRec;

static void MagickAction(Widget w, XEvent *event, String *params, Cardinal *n)
{
	;
}

static void create_ximage(Widget w)
{
	MagickWidget iw = (MagickWidget)w;

	if (iw->magick.ximage)
		XDestroyImage(iw->magick.ximage);
	if (iw->magick.image == NULL) {
		iw->magick.ximage = NULL;
	} else {
		iw->magick.ximage = XMagickImageToXImage(XtDisplay(w),
						iw->magick.image);
	}
}

#define superclass (&coreClassRec)
static void Realize(Widget w, XtValueMask *valueMask,
		XSetWindowAttributes *attributes)
{
	unsigned long valuemask = 0;
	XGCValues values;
	MagickWidget iw = (MagickWidget)w;

	(*superclass->core_class.realize)(w, valueMask, attributes);
	iw->magick.gc = XCreateGC(XtDisplay(w), XtWindow(w), valuemask, &values);
	iw->magick.ximage = NULL;
	create_ximage(w);
}

static void Destroy(Widget w)
{
	MagickWidget iw = (MagickWidget)w;
	XFreeGC(XtDisplay(w), iw->magick.gc);
	XDestroyImage(iw->magick.ximage);
}

static void Redisplay(Widget w, XEvent *event, Region r)
{
	MagickWidget iw = (MagickWidget)w;

	if (iw->magick.ximage) {
		XPutImage(XtDisplay(w), XtWindow(w), iw->magick.gc,
			iw->magick.ximage, 0, 0, 0, 0,
			iw->magick.ximage->width, iw->magick.ximage->height);
	}
}

static Boolean SetValues(Widget current, Widget request, Widget new,
		ArgList args, Cardinal *nargs)
{
	Boolean do_redisplay = True;

	create_ximage(new);

	return do_redisplay;
}

